import pytz
from datetime import datetime, time
from django.db.models import Sum
from rest_framework.generics import (
    ListAPIView,
    RetrieveAPIView,
    CreateAPIView
)
from rest_framework.views import APIView
from rest_framework.response import Response

from rest_framework.permissions import IsAuthenticated
from rest_framework.filters import SearchFilter, OrderingFilter
from mosquito.api.pagination import PostLimitOffsetPagination, PostPageNumberPagination
from .serializers import (
    DeviceCountSerializer,
    DeviceInfoSerializer,
)
from ..models import DeviceCount, DeviceInfo


class DeviceLogListAPIView(ListAPIView):
    serializer_class = DeviceCountSerializer
    permission_classes = [IsAuthenticated]
    filter_backends = [SearchFilter, OrderingFilter]
    pagination_class = PostPageNumberPagination
    search_fields = ['device_id']
    queryset = DeviceCount.objects.get_queryset().order_by('-data_time')


class DeviceInfoListAPIView(ListAPIView):
    serializer_class = DeviceInfoSerializer
    permission_classes = [IsAuthenticated]
    filter_backends = [SearchFilter, OrderingFilter]
    pagination_class = PostPageNumberPagination
    search_fields = ['device_id', 'device_name', 'chip_type', 'chip_id']
    queryset = DeviceInfo.objects.all()


class DeviceLogStatisticAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, *args, **kwargs):
        tz = pytz.timezone("UTC")
        today = datetime.now(tz).date()
        midnight = tz.localize(datetime.combine(today, time(0, 0)), is_dst=None)
        total_count = DeviceCount.objects.aggregate(total=Sum('count'))['total']
        daily_count = DeviceCount.objects.filter(
            data_time__gt=midnight).aggregate(total=Sum('count'))['total']
        data = {'total_count': total_count, 'daily_count': daily_count}
        return Response(data)


class DeviceInfoStatisticAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, *args, **kwargs):
        online_count = DeviceInfo.objects.filter(online=1).count()
        offline_count = DeviceInfo.objects.filter(online=0).count()
        data = {'online_count': online_count, 'offline_count': offline_count}
        return Response(data)
